/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/

#include <crown/CrownScheduler.hpp>
#include <crown/CrownAllocation.hpp>
#include <crown/CrownMapping.hpp>
#include <crown/CrownScaling.hpp>

#ifndef CROWN_BINARY
#define CROWN_BINARY

namespace pelib
{
namespace crown
{
	class CrownBinary : public CrownScheduler
	{
		public:
			CrownBinary(const CrownConfig *config = NULL, const CrownMapping *mapping = NULL, const CrownScaling *scaling = NULL, size_t precision = 3, bool showOutput = false, bool showError = false);
			CrownBinary(const Algebra &param, const CrownConfig *config = NULL, const CrownMapping *mapping = NULL, const CrownScaling *scaling = NULL, size_t precision = 3, bool showOutput = false, bool showError = false);
			CrownBinary(const Taskgraph &tg, const Platform &pt, const Algebra &param, const CrownConfig* = NULL, const CrownMapping *mapping = NULL, const CrownScaling *scaling = NULL, size_t precision = 3, bool showOutput = false, bool showError = false);
			CrownBinary(const CrownBinary&);
			virtual ~CrownBinary();
			Schedule schedule(const Taskgraph &tg, const Platform &pt, std::map<const string, double>&) const;
			Schedule schedule(const Taskgraph &tg, const Platform &pt, const Algebra &param, std::map<const string, double>&) const;
			Algebra solve(const Algebra &tg, const Algebra &pt, const pelib::Algebra &param, std::map<const std::basic_string<char>, double> &statistics) const;
			virtual float complexity(const Algebra &problem) const;
			virtual float complexity(const Taskgraph&, const Platform&, const Algebra&) const;
			const Algebra* solve() const;
			const Algebra* solve(map<const string, double>&) const;
			std::string getShortDescription() const;
			virtual CrownBinary* clone() const;
		protected:
			const CrownMapping *mapping;
			const CrownScaling *scaling;
			size_t precision;
		private:
			void initialize(const CrownMapping *mapping = NULL, const CrownScaling *scaling = NULL);
	};
}
}

#endif
