/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/

#include <crown/CrownComposite.hpp>
#include <crown/CrownMapping.hpp>
#include <crown/CrownScaling.hpp>

#ifndef CROWN_COMPOSITE_ANNEALING
#define CROWN_COMPOSITE_ANNEALING

namespace pelib
{
namespace crown
{
	class CrownCompositeAnnealing : public CrownComposite
	{
		public:
			CrownCompositeAnnealing(float init_temperature = default_init_temperature, float cooling_factor = default_cooling_factor, float final_temperature = default_final_temperature, float max_transformations = default_max_transformations, float max_new_states = default_max_new_states, float distance = default_distance, const CrownScheduler *coreScheduler = NULL, const CrownMapping *map = NULL, const CrownScaling *scale = NULL, bool showOutput = false, bool showError = false);
			CrownCompositeAnnealing(const Algebra &param, float init_temperature = default_init_temperature, float cooling_factor = default_cooling_factor, float final_temperature = default_final_temperature, float max_transformations = default_max_transformations, float max_new_states = default_max_new_states, float distance = default_distance, const CrownScheduler *coreScheduler = NULL, const CrownMapping *map = NULL, const CrownScaling *scale = NULL, bool showOutput = false, bool showError = false);
			CrownCompositeAnnealing(const Taskgraph &tg, const Platform &pt, const Algebra &param, float init_temperature = default_init_temperature, float cooling_factor = default_cooling_factor, float final_temperature = default_final_temperature, float max_transformations = default_max_transformations, float max_new_states = default_max_new_states, float distance = default_distance, const CrownScheduler *coreScheduler = NULL, const CrownMapping *map = NULL, const CrownScaling *scale = NULL, bool showOutput = false, bool showError = false);
			CrownCompositeAnnealing(const CrownCompositeAnnealing&);

			Schedule schedule(const Taskgraph &tg, const Platform &pt, std::map<const string, double>&) const;
			Schedule schedule(const Taskgraph &tg, const Platform &pt, const Algebra &param, std::map<const string, double>&) const;
			Algebra solve(const Algebra &tg, const Algebra &pt, const pelib::Algebra &param, std::map<const std::basic_string<char>, double> &statistics) const;
			virtual float complexity(const Algebra &problem) const;
			virtual float complexity(const Taskgraph&, const Platform&, const Algebra&) const;
			std::string getShortDescription() const;
			virtual CrownCompositeAnnealing* clone() const;

			static const float default_init_temperature;
			static const float default_cooling_factor;
			static const float default_final_temperature;
			static const float default_max_transformations;
			static const float default_max_new_states;
			static const float default_distance;

			// TODO: break into external classes	
			static float energy(const Algebra &rec);
		protected:
			Algebra neighbor(const Algebra &solution, float tasks = 1, float cores = 1) const;
			float init_temperature, cooling_factor, final_temperature, max_transformations, max_new_states, distance;
			const CrownMapping *mapping;
			const CrownScaling *scaling;
		private:
			void initialize(const CrownMapping *mapping = NULL, const CrownScaling *scaling = NULL);
	};
}
}
#endif
