/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/

#include <crown/CrownComposite.hpp>

#ifndef CROWN_COMPOSITE_CONSOLIDATION
#define CROWN_COMPOSITE_CONSOLIDATION

namespace pelib
{
namespace crown
{
	class CrownCompositeConsolidation : public CrownComposite
	{
		public:
			CrownCompositeConsolidation(const CrownScheduler *coreScheduler = NULL, bool showOutput = false, bool showError = false);
			CrownCompositeConsolidation(const Algebra &param, const CrownScheduler *coreScheduler = NULL, bool showOutput = false, bool showError = false);
			CrownCompositeConsolidation(const Taskgraph &tg, const Platform &pt, const Algebra &param, const CrownScheduler *coreScheduler = NULL, bool showOutput = false, bool showError = false);
			CrownCompositeConsolidation(const CrownCompositeConsolidation&);
			Schedule schedule(const Taskgraph &tg, const Platform &pt, std::map<const string, double>&) const;
			Schedule schedule(const Taskgraph &tg, const Platform &pt, const Algebra &param, std::map<const string, double>&) const;
			Algebra solve(const Algebra &tg, const Algebra &pt, const pelib::Algebra &param, std::map<const std::basic_string<char>, double> &statistics) const;
			virtual float complexity(const Algebra &problem) const;
			virtual float complexity(const Taskgraph&, const Platform&, const Algebra&) const;
			std::string getShortDescription() const;
			virtual CrownCompositeConsolidation* clone() const;

			static const string dummyTaskPrefix;
			static const string realTaskPrefix;
		protected:
		private:
	};
}
}
#endif
