/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/

#include <pelib/Platform.hpp>
#include <pelib/Taskgraph.hpp>
#include <pelib/Algebra.hpp>
#include <pelib/Schedule.hpp>

#include <crown/CrownConfig.hpp>
#include <crown/CrownAllocation.hpp>
#include <crown/CrownMapping.hpp>

#ifndef CROWN_SCALING
#define CROWN_SCALING

namespace pelib
{
namespace crown
{
	class CrownScaling : public CrownScheduler
	{
		public:
			CrownScaling(const CrownConfig* = NULL, const CrownAllocation *alloc = NULL, const CrownMapping *mapping = NULL, bool showOutput = false, bool showError = false);
			CrownScaling(const Algebra &param, const CrownConfig* = NULL, const CrownAllocation *alloc = NULL, const CrownMapping *mapping = NULL, bool showOutput = false, bool showError = false);
			CrownScaling(const Taskgraph &tg, const Platform &pt, const Algebra &param, const CrownConfig* = NULL, const CrownAllocation *alloc = NULL, const CrownMapping *mapping = NULL, bool showOutput = false, bool showError = false);
			CrownScaling(const CrownScaling &src);
			virtual ~CrownScaling();
			virtual Algebra scale(const Algebra&, std::map<const string, double> &stats) const = 0;
			virtual Schedule schedule(const Taskgraph &tg, const Platform &pt, std::map<const string, double>&) const;
			virtual Schedule schedule(const Taskgraph &tg, const Platform &pt, const Algebra &param, std::map<const string, double>&) const;
			Algebra solve(const Algebra &tg, const Algebra &pt, const pelib::Algebra &param, std::map<const std::basic_string<char>, double> &statistics) const;
			virtual float complexity(const Algebra &problem) const = 0;
			virtual float complexity(const Taskgraph&, const Platform&, const Algebra&, const CrownConfig* = NULL) const = 0;
			virtual std::string getShortDescription() const = 0;
			virtual CrownScaling* clone() const = 0;
		protected:
			const CrownAllocation *alloc;
			const CrownMapping *mapping;
		private:
			void initialize(const CrownAllocation * = NULL, const CrownMapping * = NULL);
	};
}
}

#endif

