/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/

#include <crown/CrownScaling.hpp>
#include <pelib/Platform.hpp>
#include <pelib/Taskgraph.hpp>
#include <pelib/Algebra.hpp>

#ifndef CROWN_SCALING_ILP
#define CROWN_SCALING_ILP

namespace pelib
{
namespace crown
{
	class CrownScalingILP: public CrownScaling
	{
		public:
			CrownScalingILP(const CrownConfig* = NULL, const CrownAllocation *alloc = NULL, const CrownMapping *mapping = NULL, bool showOutput = false, bool showError = false);
			CrownScalingILP(const Algebra &param, const CrownConfig* = NULL, const CrownAllocation *alloc = NULL, const CrownMapping *mapping = NULL, bool showOutput = false, bool showError = false);
			CrownScalingILP(const Taskgraph &tg, const Platform &pt, const Algebra &param, const CrownConfig* = NULL, const CrownAllocation *alloc = NULL, const CrownMapping *mapping = NULL, bool showOutput = false, bool showError = false);
			CrownScalingILP(bool showOutput = false, bool showError = false);
			virtual Algebra scale(const Algebra&, std::map<const string, double> &stats) const;
			virtual Schedule scale(const Taskgraph&, const Platform&, const Algebra&, const CrownConfig *config, std::map<const string, double> &stats) const;
			std::string getShortDescription() const;
			virtual float complexity(const Algebra &problem) const;
			virtual float complexity(const Taskgraph&, const Platform&, const Algebra&, const CrownConfig* = NULL) const;
			virtual CrownScaling* clone() const;
		protected:
	};
}
}

#endif

