/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/

#include <pelib/Scheduler.hpp>
#include <pelib/Solver.hpp>
#include <pelib/Algebra.hpp>
#include <pelib/Taskgraph.hpp>
#include <pelib/Platform.hpp>
#include <pelib/Schedule.hpp>
#include <pelib/constraints.hpp>

#include <crown/CrownConfig.hpp>

#ifndef CROWN_SCHEDULER
#define CROWN_SCHEDULER

namespace pelib
{
namespace crown
{
	class CrownScheduler : public Scheduler
	{
		public:
			CrownScheduler(const CrownConfig* = NULL, bool showOutput = false, bool showError = false);
			CrownScheduler(const Algebra &param, const CrownConfig* = NULL, bool showOutput = false, bool showError = false);
			CrownScheduler(const Taskgraph &tg, const Platform &pt, const Algebra &param, const CrownConfig* = NULL, bool showOutput = false, bool showError = false);
			CrownScheduler(const CrownScheduler &src);
			virtual ~CrownScheduler();
			virtual Schedule schedule(const Taskgraph &tg, const Platform &pt, std::map<const string, double>&) const = 0;
			virtual Schedule schedule(const Taskgraph &tg, const Platform &pt, const Algebra &param, std::map<const string, double>&) const = 0;
			virtual float complexity(const Algebra &problem) const;
			virtual float complexity(const Taskgraph&, const Platform&, const Algebra&) const;
			const Algebra* solve() const;
			const Algebra* solve(map<const string, double>&) const;
			virtual Algebra solve(const Algebra &tg, const Algebra &pt, const pelib::Algebra &param, std::map<const std::basic_string<char>, double> &statistics) const = 0;
			virtual CrownScheduler* clone() const = 0;
			virtual std::string getShortDescription() const;
			// TODO: make it protected
			static Algebra crownToSchedule(const Algebra &crown);
			const CrownConfig* getCrownConfig() const;
			const CrownConfig *config;
			Algebra addCrownConfig(const Taskgraph&, const Platform&, const Algebra&, std::map<const string, double>&) const;
		protected:
			CrownConfig* getDefaultConfig() const;
			bool showError, showOutput;
			Taskgraph tg;
			Platform pt;
			Algebra param;
		private:
			void initialize(const CrownConfig* = NULL);
	};
}
}

#endif
