/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <pelib/Algebra.hpp>

#ifndef CROWN_H
#define CROWN_H

// Name of a dummy task not to be taken into account when computing energy consumption
#define DUMMY_TASK "Unused Cores"

namespace pelib
{
namespace crown
{

#if 0
/** Run simulated annealing to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and the "simple" energy model. **/
Algebra
crown_annealing_allocation_simple(const Algebra &initial, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Run simulated annealing to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model including energy consumed by processors at idle time. **/
Algebra
crown_annealing_allocation_idle(const Algebra &initial, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Run simulated annealing to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model excluding the energy consumed by processors at idle time. **/
Algebra
crown_annealing_allocation_off(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Run binary search to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and the "simple" energy model. **/
Algebra
crown_binary_simple(const Algebra &schedule, unsigned int precision);

/** Run binary search to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model including energy consumed by processors at idle time. **/
Algebra
crown_binary_idle(const Algebra &schedule, unsigned int precision);

/** Run binary search to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model excluding the energy consumed by processors at idle time. **/
Algebra
crown_binary_off(const Algebra &schedule, unsigned int precision);
#endif

/** Run binary search to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and a generic energy model depending on scalars alpha, alpha-static, zeta and epsilon. **/
Algebra
crown_binary(const Algebra &schedule, unsigned int precision);

Algebra
crown_binary_annealing_allocation(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Return the complexity in number of (virtual) instructions of binary search to compute an allocation for an input problem **/
float
crown_binary_complexity(const Algebra &schedule, unsigned int precision = 0);

#if 0
/** Run binary search and simulated annealing to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and the "simple" energy model. **/
Algebra
crown_binary_annealing_allocation_simple(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);
	
/** Run binary search and simulated annealing to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model including energy consumed by processors at idle time. **/
Algebra
crown_binary_annealing_allocation_idle(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Run binary search and simulated annealing to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model excluding the energy consumed by processors at idle time. **/
Algebra
crown_binary_annealing_allocation_off(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Run binary search and simulated annealing on task minimal efficiency, to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and the "simple" energy model. **/
Algebra
crown_binary_annealing_efficiency_simple(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Run binary search and simulated annealing on task minimal efficiency, to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model including energy consumed by processors at idle time. **/
Algebra
crown_binary_annealing_efficiency_idle(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);

/** Run binary search and simulated annealing on task minimal efficiency, to compute allocation, using LTLG and Height heuristics for mapping and frequency scaling, respectively, and an energy model excluding the energy consumed by processors at idle time. **/
Algebra
crown_binary_annealing_efficiency_off(const Algebra &schedule, float temperature, float cooling, float final, int max_transform, int max_new_state, float distance);
#endif
	
}
}

#endif
