/*
 Copyright 2015 Nicolas Melot

 This file is part of Pelib.

 Pelib is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Pelib is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Pelib. If not, see <http://www.gnu.org/licenses/>.
*/

#include <pelib/Algebra.hpp>
#include <pelib/Scalar.hpp>
#include <pelib/AmplSolver.hpp>
#include <pelib/Vector.hpp>
#include <pelib/Matrix.hpp>
#include <pelib/AmplInput.hpp>

#include <crown/CrownModular.hpp>
#include <crown/CrownComposite.hpp>
#include <crown/CrownAllocationFastest.hpp>
#include <crown/CrownMappingLTLG.hpp>
#include <crown/CrownScalingHeight.hpp>
#include <crown/CrownConfigBinary.hpp>
#include <crown/CrownException.hpp>

#include <crown/mapping.h>
#include <crown/scaling.h>

#ifdef debug
#undef debug
#endif

#define debug(var) cout << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "] " << #var << " = \"" << (var) << "\"" << endl;

using namespace std;
using namespace pelib;
using namespace crown;

void
CrownComposite::initialize(const CrownScheduler *scheduler)
{
	if(scheduler == NULL)
	{
		this->scheduler = new CrownModular(NULL, NULL, NULL, NULL, showOutput, showError);
	}
	else
	{
		this->scheduler = scheduler->clone();
	}
}

CrownComposite::CrownComposite(const CrownScheduler *scheduler, bool showOutput, bool showError) : CrownScheduler(NULL, showOutput, showError)
{
	initialize(scheduler);

	// TODO: restore arbitrary configuration
	//delete this->config;
	//this->config = new CrownConfigBinary();
}

CrownComposite::CrownComposite(const Algebra &param, const CrownScheduler *scheduler, bool showOutput, bool showError) : CrownScheduler(param, NULL, showOutput, showError)
{
	initialize(scheduler);

	// TODO: restore arbitrary configuration
	//delete this->config;
	//this->config = new CrownConfigBinary();
}

CrownComposite::CrownComposite(const Taskgraph &tg, const Platform &pt, const Algebra &param, const CrownScheduler *scheduler, bool showOutput, bool showError) : CrownScheduler(tg, pt, param, NULL, showOutput, showError)
{
	initialize(scheduler);

	// TODO: restore arbitrary configuration
	//delete this->config;
	//this->config = new CrownConfigBinary();
}

CrownComposite::CrownComposite(const CrownComposite &src) : CrownScheduler(src)
{
	initialize(src.scheduler);
	//delete this->config;
	//this->config = new CrownConfigBinary();
}

CrownComposite::~CrownComposite()
{
	delete this->scheduler;
}

string
CrownComposite::getShortDescription() const
{
	return string("Comp.") + this->scheduler->getShortDescription();
}

