/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <fstream>

#include <pelib/argument_parsing.hpp>
#include <pelib/dl.h>

#include <crown/allocation.h>
#include <crown/CrownAllocationFastest.hpp>
#include <crown/CrownConfigBinary.hpp>
#include <crown/CrownConfigOrgan.hpp>
#include <crown/CrownConfigTaskgraph.hpp>

#ifdef debug
#undef debug
#endif

#define debug(var) cout << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "] " << #var << " = \"" << (var) << "\"" << endl;

using namespace std;
using namespace pelib;
using namespace crown;

#ifdef __cplusplus
extern "C" {
#endif

struct args
{
	float gemin;
	bool showError, showOutput;
	
};
typedef struct args args_t;

static args_t
parse(char **args)
{
	args_t out;
	out.showError = false;
	out.showOutput = false;
	out.gemin = 0;

	for(; args[0] != NULL; args++)
	{
		if(strcmp(args[0], "--show-stdout") == 0)
		{
			out.showOutput = true;
			continue;
		}
		if(strcmp(args[0], "--show-stderr") == 0)
		{
			out.showError = true;
			continue;
		}
		if(strcmp(args[0], "--min-efficiency") == 0)
		{
			stringstream str(args[0]);
			str >> out.gemin;
			continue;
		}
	}

	return out;
}

const CrownAllocation*
crown_allocation(size_t argc, char **argv)
{
	args_t args = parse(argv);

	// Convert solver output to general schedule
	CrownAllocation *alloc = new CrownAllocationFastest(args.gemin, args.showOutput, args.showError);
	return alloc;
}

void
crown_delete_algebra(pelib::Algebra *alloc)
{
	delete alloc;
}

#ifdef __cplusplus
}
#endif
