/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <fstream>

#include <pelib/argument_parsing.hpp>
#include <pelib/scheduler.h>
#include <pelib/dl.h>

#include <crown/CrownScheduler.hpp>
#include <crown/crown-scheduler.hpp>

#ifdef debug
#undef debug
#endif

#define debug(var) cout << "[" << __FILE__ << ":" << __FUNCTION__ << ":" << __LINE__ << "] " << #var << " = \"" << (var) << "\"" << endl;

using namespace std;
using namespace pelib;
using namespace crown;

#ifdef __cplusplus
extern "C" {
#endif

const pelib::Schedule*
pelib_schedule(const pelib::Taskgraph &tg, const pelib::Platform &pt, size_t argc, char **argv, map<const string, double>& statistics)
{
	const CrownScheduler *scheduler = crown_scheduler(argc, argv, NULL);
	return new Schedule(scheduler->schedule(tg, pt, statistics));
}

string
pelib_description(size_t argc, char **argv)
{
	const CrownScheduler *sched = crown_scheduler(argc, argv, NULL);
	string desc = sched->getShortDescription();
	return desc;
}

void
pelib_delete(pelib::Schedule *sched)
{
	delete sched;
}

void
crown_delete_scheduler(const pelib::crown::CrownScheduler *sched)
{
	delete sched;
}

#ifdef __cplusplus
}
#endif
