### Platform
param p;
set P = 1..p;
set F;
param Funit;
## Cannot have more than 2p - 1 recursive groups, no matter what the recursive
## decomposition is applied, even with asymetric and irregular decompositions.
param q = 2 * p - 1;
set G = 1..q;

## Crown tuning
param alpha; ## Dynamic power of a frequency
param zeta; # Weight for static energy consumption
param kappa; # Constant part of static power
param eta; # Idle power
param crown_configuration_groups{g in G, pp in P} binary;
param crown_configuration_tree{g in G, gg in G} binary;
## Frequency island extension
param Fin;
set FiS = 1 .. Fin;
set Fi{i in FiS};

param Sin;
set Si{i in 1..Sin};

## Taskgraph & deadline
param M; # Crown maximal makespan
param n; # Number of tasks
set T = 1..n; # Task set
param Wi{t in T}; # Task width
param Tau{t in T}; # Task work
param sTau{t in T}; # Task work
param e{t in T, pp in P}; # Task efficiency distribution
param c{t in T, tt in T}; # Task channels
param streaming{t in T};

## Energy model
param busy_power{f in F} = (zeta * f^(alpha) + (1 - zeta) * (f + kappa * min{ff in F}ff)) / zeta;
param idle_power = eta * min{f in F}busy_power[f];

## Takes value 1 if group i owns processor j;
param member{g in G, pp in P} binary = crown_configuration_groups[g, pp];
#var member{g in G, pp in P} binary;
## if 1, then j is precedence of i
param precedence{g in G, gg in G} binary = crown_configuration_tree[g, gg];
#var precedence{g in G, gg in G} binary;

## All cores of group k must be included in the precedence i containing k
#subject to crown{g in G, pp in P, gg in G: g < gg}: (1 - member[gg, pp]) + (1 - member[g, pp]) * (1 - precedence[gg, g]) + (member[g, pp] * precedence[gg, g]) >= 1; ## A subgroup must not be as large as its precedence

param size{g in G} = sum{pp in P} member[g, pp];
set GmC{g in G} = union{pp in P} if member[g, pp] then {pp} else {};
set Fii{pp in P} = union{i in FiS} if pp in Fi[i] then {i} else {};
set GmI{g in G} = union{pp in GmC[g]} Fii[pp];
set Gs{g in G} = union{gg in G: gg != g} if card(inter{ggg in {g, gg}} GmI[ggg]) > 0 then {gg} else {};
#set Gii{g in G} = union{i in FiS} if inter{pp in Fi[i]};
#set Gs{g in G} = union{gg in G} if card(inter{pp in (g,gg)}, GmC[gg]) > 0 then {gg} else {}

#var size{g in G} integer;
#	subject to stuff{g in G}: size[g] = sum{pp in P} member[g, pp];
## Groups that match exactly one or more frequency islands
set Gi = union{g in G} if card(union{i in FiS} if GmC[g] within Fi[i] and Fi[i] within GmC[g] then {i} else {}) > 0 then {g} else {};
## Subgroups of a group
set sG{g in G} = union{gg in G} if precedence[gg, g] then {gg} else {};
## Make sure than every group span over less than an island, or an exact multiple of islands, unless the group is part of only one island
	subject to groups_match_islands{g in G}: (GmC[g] within union{i in GmI[g]} Fi[i] and union{i in GmI[g]} Fi[i] within GmC[g]) or card(GmI[g]) = 1;

#subject to decompose{g in G, gg in G: g < gg}: size[gg] * precedence[gg, g] <= size[g] - precedence[gg, g];
param size_bin{g in G, pp in P} binary = if pp == size[g] then 1 else 0;
#var size_bin{g in G, pp in P} binary;
#subject to core_set_is_number_of_cores{g in G}: sum{pp in P} size_bin[g, pp] * pp = size[g];
#subject to only_one_core_number_per_group{g in G}: sum{pp in P} size_bin[g, pp] = 1;

## Constraints the solution to what is given in groups parameter
#subject to predefined_groups{g in G, pp in P}: member[g, pp] >= groups[g, pp];
#subject to predefined_tree{g in G, gg in G}: precedence[g, gg] >= tree[g, gg];

## Setting matrix. task t runs at frequency f on size[g] core iff x[g, t, f] = 1
var x{g in G, t in T, k in F} binary;
	## A task cannot be mapped to a group of more cores than the task's maximum width. Is it really necessary?
	subject to no_mapping_to_groups_bigger_than_task_max_width{t in T}: sum{g in G, f in F} x[g, t, f] * size[g] <= Wi[t];
	## A task can only receive one group and frequency setting
	subject to one_setting_per_task{t in T}: sum{g in G, f in F} x[g, t, f] = 1;
	#subject to frequency_island{g in Gi, gg in Gs[g], t in T, tt in T, k in F: t != tt and size[gg] < size[g]}: x[gg, t, k] = x[gg, tt, k];

var iF{g in Gi, f in F} binary;
	subject to task_mapped_to_non_f{g in Gi, f in F}: iF[g, f] >= sum{gg in sG[g], t in T, ff in F: ff != f} x[gg, t, ff] / n; ## Forces iF to 1 if at least one task mapped to subgroups of Gi is not set to f
	subject to exactly_one_frequency_tasks_mapped_to_non_other{g in Gi}: sum{f in F} iF[g, f] = card(F) - 1; ## Make sure there is exactly one f in F for each g in Gi, such that Fi[f] = 0, i.e., for each group-island, there is exactly one frequency level such that no tasks mapped to any proper subgroup of g, runs at another frequency than f.

# Binary var product trick
# z = x * y <=> z - x <= 0, z - y <= 0, -z + x + y <= 1 

## u[pp] is 1 if the core runs at least a task
var u{pp in P} binary;
#subject to zero_if_unused{pp in P}: u[pp] <= sum{g in G} member[g, pp] * sum{t in T} sum{f in F} x[g, t, f];
subject to one_if_used{pp in P, t in T, f in F}: sum{g in G} member[g, pp] * u[pp] >= sum{g in G} member[g, pp] * x[g, t, f];
subject to shutdown_whole_island{i in FiS, pp in Fi[i], ppp in Fi[i]: pp != ppp}: u[pp] = u[ppp];

## Running time of core pp
var core_time{pp in P} = sum{t in T, g in G, f in F, ppp in P} x[g, t, f] * member[g, pp] * size_bin[g, ppp] * Tau[t] / (ppp * e[t, ppp] * f);
var time_busy = sum{pp in P} core_time[pp];

## Idle time of core pp, 0 if the core is not used at all (switched off)
var core_idle_time = ((M * (card(P) - sum{i in P} (1 - u[i]))) - time_busy);

## Every core must not run for more than M
subject to meet_deadline{pp in P}: core_time[pp] <= M;

## Energy consumption
## Busy energy
var busy_energy_task{t in T} = sum{g in G, f in F, pp in P} size_bin[g, pp] * pp * x[g, t, f] * busy_power[f] * Tau[t] / (e[t, pp] * pp * f);
var time_task{t in T} = sum{g in G, f in F, pp in P} size_bin[g, pp] * x[g, t, f] * Tau[t] / (e[t, pp] * pp * f);
var busy_energy = sum{g in G, t in T, f in F, pp in P} size_bin[g, pp] * pp * x[g, t, f] * busy_power[f] * Tau[t] / (e[t, pp] * pp * f);
#param busy_energy = 0;
## Idle energy
var idle_energy = core_idle_time * idle_power;
#param idle_energy = 0;
## Final energy
var energy = zeta * busy_energy + (1 - zeta) * idle_energy;

## Minimize energy
minimize target: energy;

## Helper variables
## Task's frequency
var frequency{t in T} = sum{g in G, f in F} x[g, t, f] * f;
## Task's group
var _group{t in T} = sum{g in G, f in F} x[g, t, f] * g;
var group{t in T} = round(_group[t]);
## Task's allocated width
var wi{t in T} = sum{g in G, f in F} x[g, t, f] * size[g];

## Debugging variables
var task_energy{t in T} = sum{g in G, f in F, pp in P} size_bin[g, pp] * pp * x[g, t, f] * busy_power[f] * Tau[t] / (e[t, pp] * pp * f);
var task_all_energy{t in T, f in F} = sum{g in G, pp in P, ff in F} size_bin[g, pp] * pp * x[g, t, ff] * busy_power[f] * Tau[t] / (e[t, pp] * pp * f);
var task_power{t in T, f in F} = sum{g in G, pp in P, ff in F} size_bin[g, pp] * x[g, t, ff] * busy_power[f];
var task_surface{t in T, f in F} = sum{g in G, pp in P, ff in F} size_bin[g, pp] * pp * x[g, t, ff] * (Tau[t] / (e[t, pp] * pp * f));
var task_freq{t in T, f in F} = sum{g in G, pp in P, ff in F} size_bin[g, pp] * x[g, t, ff] * f;

